/**
 * Ana_Wavelet nt= dt= ot= frq= > wavelet.H
 */

#include <math.h>
#include <seplib.h>

#define MY_SEP_DOC \
sep_add_doc_line("NAME");\
sep_add_doc_line("    Ana_Wavelet - Generate ricker wavelet and its derivative");\
sep_add_doc_line("                  from the analytic formulae");\
sep_add_doc_line(""); \
sep_add_doc_line("SYNOPSIS");\
sep_add_doc_line("    Ana_Wavelet nt= [dt= ot= frq= der= dly= ] > wav.H");\
sep_add_doc_line(""); \
sep_add_doc_line("INPUT PARAMETERS");\
sep_add_doc_line("    nt - int");\
sep_add_doc_line("            number of time samples on wavelet");\
sep_add_doc_line(""); \
sep_add_doc_line("More to come...");\
sep_add_doc_line(""); \

int main(int argc, char **argv) {

  int nt;
  int it;
  int der;
  float ot, dt;
  float frq, dly;

  /* Initialize I/O */
  initpar(argc, argv);

  /* Generate self doc */
  MY_SEP_DOC; doc(SOURCE);

  /* Get commandline args */
  if(!getch("nt","d",&nt)) seperr("Could not get n1\n");
  dt  = 0.004; getch("dt","f",&dt );
  ot  = 0.0;   getch("ot","f",&ot );
  der = 0;     getch("der","d",&der);
  frq = 30.0;  getch("frq","f",&frq);
  dly = 1.0;   getch("dly","f",&dly);

  fprintf(stderr,"nt=%d dt=%f ot=%f dly=%f\n",nt,dt,ot,dly);

  float* wav = malloc(nt*sizeof*wav);

  if (!der) {
    for(it = 0; it < nt; ++it) {
      float t    = it * dt - dly;
      float pift = M_PI * M_PI * (frq * frq) * (t * t);
      wav[it]    = (1 - 2 * pift) * (exp(-pift));
    }
  } else {
    for(it = 0; it < nt; ++it) {
      float t    = it * dt - dly;
      float pif  = M_PI * M_PI * (frq * frq);
      float pift = M_PI * M_PI * (frq * frq) * (t * t);
      wav[it] = (2 * pift - 3) * 2 * t * pif * exp(-pift);
    }
  }

  /* Write values to file */
  if(putch("n1","d",&nt)) seperr("Could not write n1 to file");
  if(putch("d1","f",&dt)) seperr("Could not write d1 to file");
  if(putch("o1","f",&ot)) seperr("Could not write o1 to file");
  if(putch("tdelay","f",&dly)) seperr("Could not write tdelay to file");

  /* Write out wavelet */
  if(!srite("out",wav,sizeof*wav*nt))
    seperr("trouble writing dataset");

  /* Free up memory */
  free(wav);

  return 0;
}
