program Inv_tdfwi

  use sep
  use axis_mod
  use mem_mod
  use Util1Daxis_mod
  use prop1Daxis_mod
  use prop_der11Daxis_mod
  use prop_extend_der11Daxis_mod
  use prop_extend1Daxis_mod
  use focus1Daxis_mod
  use prob_nl_l2_diff_ext_mod
  use prob_l_l2_reg_mod
  use prob_comb_sum_mod
  use mysample_mod
  use myparab_mod
  use stepper_exhaust_mod
  use nlsolver_mod

  implicit none

  ! Axes
  type(axis)                          :: adat, awav, aslw, atau
  integer                             :: zlag, plag
  ! Flags
  logical                             :: verb,mov
  logical                             :: use_foc,foc_comp
  ! Inversion/modeling parameters
  real                                :: dly
  real                                :: dist
  real                                :: eps
  integer                             :: niter
  ! Counters
  integer                             :: i_slow
  integer                             :: stat
  ! Inversion objects
  type(prob_nl_l2_diff_ext)           :: prob_diff
  type(prob_l_l2_reg)                 :: prob_reg
  type(prob_comb_sum)                 :: prob_comb
  type(mysample)                      :: smp
  type(myparab)                       :: prb
  type(stepper_exhaust)               :: exh

  ! Model, data and wavelet arrays
  real, dimension(:), allocatable     :: dat,mdl,wav,der

  ! Timer
  integer(kind=8)                     :: timer0, timer1, rate
  real                                :: timer_sum

  ! Movie file names
  character(256)                      :: rstr,gstr,mstr,nstr,ext
  character(256)                      :: rmov,gmov,mmov,rnrm
  character(256)                      :: alphaplot,alphastr


  call sep_init()

 !---------------------------------------------------------------------------
  call system_clock(timer0, rate)
 !---------------------------------------------------------------------------

  ! Get command line parameters
  call from_param("eps", eps, 0.01)
  call from_param("niter",niter,100)
  call from_param("dist", dist, 2.0)
  call from_param("verb", verb, .false.)
  call from_param("mov", mov, .true.)
  call from_param("use_focus_oper", use_foc, .false.)
  call from_param("focus_complement", foc_comp, .true.)

  !==================== Get/Put axes =================!
  ! wavelet axis
  call get_wavelet_axis(awav,dly,"wav")

  ! data axis
  adat = get_data_axis("in")

  ! Slowness axis
  aslw = get_backslow_axis()

  ! tau axis
  atau = get_slow_axis()
  zlag = calc_zlag(atau)
  plag = atau%n
  call put_slow_axis(atau, "out", aslw)

  !==================== Allocate memory =================!
  call mem_alloc(wav, awav%n)
  call mem_alloc(der, awav%n)
  call mem_alloc(dat, adat%n)
  call mem_alloc(mdl, atau%n)
  call sreed("wav", wav, 4*awav%n)
  call sreed("der", der, 4*awav%n)

  !=================== Operators Init ===================!
  call prop1Daxis_init            (adat, awav,       dly, dist     )
  call prop_der11Daxis_init       (adat, awav,       dly, dist, der)
  call prop_extend_der11Daxis_init(adat, awav, atau, dly, dist, der)
  call prop_extend1Daxis_init     (adat, awav, atau, dly, dist, wav)
  call focus1Daxis_init(verb, use_foc, foc_comp, atau, zlag)
  stat = prop1Daxis_wavelet_update(wav)

  rmov = 'TDrmov'; gmov = 'TDgmov'; mmov = 'TDmmov'; rnrm = 'TDrnrm'
  alphaplot = 'TDalpha'
  ext  = '.H'

  !=============== Run inversion for range of s0=============!
  do i_slow = 1, aslw%n
    call sep_read(mdl,"C0")
    mdl(plag) = aslw%o + (i_slow - 1) * aslw%d
    if(verb) write(0,*) "s0= ",mdl(plag)
    call sep_read(dat)
    ! Create problem
    prob_diff = prob_nl_l2_diff_ext(prop1Daxis_nlop,prop_extend1Daxis_op,         &
                                    prop_der11Daxis_phys_slow_update,             &
                                    prop_extend_der11Daxis_extnd_slow_update,     &
                                    prop_extend_der11Daxis_back_slow_update,      &
                                    prop_extend1Daxis_back_slow_update,           &
                                    prop_der11Daxis_op,prop_extend_der11Daxis_op, &
                                    atau%n,adat%n,dat)
    prob_reg = prob_l_l2_reg(focus1Daxis_op,atau%n,atau%n)
    prob_comb = prob_comb_sum(prob_diff,prob_reg,eps)
    ! Movies
    if(mov) then
      write(rstr,"(A6,F4.2,A2)") rmov,mdl(plag),ext
      write(gstr,"(A6,F4.2,A2)") gmov,mdl(plag),ext
      write(mstr,"(A6,F4.2,A2)") mmov,mdl(plag),ext
      write(nstr,"(A6,F4.2,A2)") rnrm,mdl(plag),ext
      write(alphastr,"(A7,F4.2,A2)") alphaplot,mdl(plag),ext
      prob_comb%rmov  = rstr
      prob_comb%gmov  = gstr
      prob_comb%mmov  = mstr
      !prob_comb%rnorm = nstr
      prob_diff%rnorm = nstr
    endif
    ! Create steppers
    prb = myparab (ntry=20,alpha=0.0)
    smp = mysample(ntry=100,alpha=0.0)
    exh = stepper_exhaust(na=50000,da=0.0000001,oa=0.0, &
                            plot=alphastr,verb=.false.,time=.true.)
    ! Run solver
    stat  = nlsolver(modl=mdl,prob=prob_comb,nsolver=NLCG,niter=niter,stepper=prb)
    write(0,*) "estimated s= ",mdl(plag)
    call sep_write(mdl)
  enddo

 !---------------------------------------------------------------------------
  call system_clock(timer1)
  timer_sum = 1.*(timer1-timer0)/rate
  call to_history("timer sum (min)", timer_sum/60.)
  write(0,*) "timer sum (min)", timer_sum/60.
 !---------------------------------------------------------------------------

 !=============== Clean up =====================!
  call mem_dealloc(wav)
  call mem_dealloc(der)
  call mem_dealloc(dat)
  call mem_dealloc(mdl)

end program
