program JWF1D

  use sep
  use mem_mod
  use prop_extend1D_mod
  use prop1D_mod
  use focus1D_mod
  use Util1D_mod

  implicit none

  logical, parameter                  :: T = .true., F = .false.
  logical                             :: verb,adj
  logical                             :: use_focus_oper,focus_complement,shrink
  logical                             :: target_image

  integer                             :: t_n,wavelet_n
  integer                             :: lag_n,slow_n,slow_n_data,i_slow
  integer                             :: back_lag_n,back_slow_n,i_back_slow
  integer                             :: focus_oper_n,focus_oper_i,interp_n

  integer                             :: zero_lag,slow_phys_lag
  integer                             :: back_zero_lag,back_slow_phys_lag
  integer                             :: ret_seek,sseek

  real                                :: lag_d,lag_o,slow_o,slow_d
  real                                :: back_lag_d,back_lag_o
  real                                :: slow_o_data,slow_d_data
  real                                :: wavelet_o,wavelet_d,t_o,t_d
  real                                :: wavelet_tdelay
  real                                :: slow_phys,back_slow_phys,prop_dist
  real                                :: focus_oper_o,focus_oper_d
  real                                :: focus_strength

  real, dimension(:), allocatable     :: wavelet
  real, dimension(:), allocatable     :: slow
  real, dimension(:), allocatable     :: slow_conv_defoc,slow_conv_foc
  real, dimension(:), allocatable     :: slow_conv_diff
  real, dimension(:), allocatable     :: data_rec,data_mod,data_res
  real, dimension(:,:), allocatable   :: J,slow_conv

  integer(kind=8)                     :: timer0, timer1, rate
  real                                :: timer_sum

  call sep_init()

!---------------------------------------------------------------------------
  call system_clock(timer0, rate)
!---------------------------------------------------------------------------

  call from_param("use_focus_oper", use_focus_oper, T)
  call from_param("focus_complement", focus_complement, T)
  call from_param("target_image",target_image, F)
  call from_param("shrink",shrink, F)
  call from_param("focus_strength", focus_strength, .0001)
  call from_param("interp_n", interp_n, 9)
 
  if(focus_complement .and. target_image) &
    call seperr("focus_complemet should be false with target_image true")
  if((.not. focus_complement) .and. (.not. target_image)) &
    call seperr("focus_complemet should be false with target_image true")


  call get_process_param(verb,adj)
  call get_acq_param(prop_dist)
  call get_data_param(t_n,t_o,t_d,slow_n_data,slow_o_data,slow_d_data,"in")
  call get_slow_param(lag_n,lag_o,lag_d,slow_phys_lag,zero_lag)
  call get_back_slow_param(back_lag_n,back_lag_o,back_lag_d, &
         back_slow_phys_lag,back_zero_lag, &
         slow_n,slow_o,slow_d,"Slow_back")
  call get_wavelet_param(wavelet_n,wavelet_o,wavelet_d,wavelet_tdelay,"Wavelet")

  if(use_focus_oper) then
    call get_focus_oper_param( &
       focus_oper_n,focus_oper_o,focus_oper_d,"focus_oper")
  else
    call get_focus_oper_param( &
       focus_oper_n,focus_oper_o,focus_oper_d)
!       focus_oper_n=1
!       focus_oper_o=0.
!       focus_oper_d=1.
  end if

  if(slow_n .ne. slow_n_data) call seperr("slow_n /= slow_n_data")
  if(slow_o .ne. slow_o_data) call seperr("slow_o /= slow_o_data")
  if(slow_d .ne. slow_d_data) call seperr("slow_d /= slow_d_data")

  call to_history("n1", slow_n, "out")
  call to_history("o1", slow_o, "out")
  call to_history("d1", slow_d, "out")
  call to_history("label1", "Slowness (s/km)", "out")

  if (target_image) then
    back_slow_n=slow_n
    call to_history("n2", back_slow_n, "out")
    call to_history("o2", slow_o, "out")
    call to_history("d2", slow_d, "out")
    call to_history("label2", "Background Slowness (s/km)", "out")

    call to_history("n3", focus_oper_n, "out")
    call to_history("o3", focus_oper_o, "out")
    call to_history("d3", focus_oper_d, "out")
    call to_history("label3", "Focus_oper (#)", "out")

  else
    back_slow_n=1

    call to_history("n2", focus_oper_n, "out")
    call to_history("o2", focus_oper_o, "out")
    call to_history("d2", focus_oper_d, "out")
    call to_history("label2", "Focus_oper (#)", "out")
  end if

  call prop1D_init(t_n,wavelet_n,lag_n,lag_o,lag_d, &
      wavelet_o,wavelet_d,t_o,t_d,wavelet_tdelay,prop_dist)

  call prop_extend1D_init(t_n,wavelet_n,lag_n,lag_o,lag_d, &
      back_lag_n,back_lag_o,back_lag_d, &
      wavelet_o,wavelet_d,t_o,t_d,wavelet_tdelay,prop_dist)

  if(use_focus_oper) then
    call focus1D_init(verb,use_focus_oper,focus_complement,shrink, &
           lag_n,slow_phys_lag,zero_lag,lag_o,lag_d, &
           focus_oper_n,focus_oper_o,focus_oper_d, &
           interp_n,focus_strength)
  else
    call focus1D_init(verb,use_focus_oper,focus_complement,shrink, &
           lag_n,slow_phys_lag,zero_lag,lag_o,lag_d, &
           focus_oper_n,focus_oper_o,focus_oper_d, &
           interp_n)
  end if

  call mem_alloc(wavelet, wavelet_n)
  call mem_alloc(data_rec, t_n)
  call mem_alloc(data_mod, t_n)
  call mem_alloc(data_res, t_n)
  call mem_alloc(slow, slow_n)
  call mem_alloc(slow_conv_defoc, lag_n)
  call mem_alloc(slow_conv_foc, lag_n)
  call mem_alloc(slow_conv_diff, lag_n)
  call mem_alloc(slow_conv, lag_n,slow_n)
  call mem_alloc(J, slow_n,back_slow_n)

  call sep_read(wavelet,"Wavelet")
  call prop_extend1D_wavelet_update2(wavelet)

  do focus_oper_i=1,focus_oper_n
  !{
    ret_seek=sseek("Slow_back",0,0)
    call sep_read(slow,"Slow_back")

    call focus1D_oper_update2(focus_oper_i)

    ret_seek=sseek("in",0,0)
    do i_slow=1,slow_n
    !{
      call sep_read(data_rec)
      slow_phys=slow(i_slow)
!      if(verb) write(0,*) "i_slow=",i_slow,"slow_phy=",slow_phys
      call prop1D_slow_update2(slow_phys)
      call prop_extend1D_back_slow_update2(slow_phys)

      call prop1D_op2(F, F, wavelet, data_mod)
      data_res=(data_mod-data_rec)
!call snap("Res.H",t_n,1,data_res)

      call prop_extend1D_op2(T, F, slow_conv(:,i_slow), data_res)
      if(.not. target_image) then
        call focus1D_op2(F, F, slow_conv(:,i_slow), slow_conv_defoc)
!write(0,*)'slow_conv_defoc(zero_lag)=',slow_conv_defoc(zero_lag)
        J(i_slow,back_slow_n) =  &
          .5*dot_product(slow_conv_defoc,slow_conv_defoc)
      end if
    !}
    end do

    if(target_image) then
    !{
      do i_back_slow=1,back_slow_n
      !{
write(0,*)'focus_oper_i=',focus_oper_i,'i_back_slow=',i_back_slow

        call focus1D_op2(F, F, slow_conv(:,i_back_slow), slow_conv_foc)
!        back_slow_phys=slow(i_back_slow)
!        call prop_extend1D_back_slow_update2(back_slow_phys)
!        call prop_extend1D_op2(T, F, slow_conv, data_res)
        do i_slow=1,slow_n
        !{
          slow_conv_diff=slow_conv(:,i_slow)-slow_conv_foc 
          J(i_slow,i_back_slow) =  &
            .5*dot_product(slow_conv_diff,slow_conv_diff)
        !}
        end do
      !}
      end do
    !}
    end if
    call sep_write(J)
  !}
  end do

!      else
!        call focus1D_op2(F, F, slow_conv, slow_conv_defoc)
!        J(i_slow,back_slow_n) =  &
!          .5*dot_product(slow_conv_defoc,slow_conv_defoc)
!      end if
!!call snap("Slow_conv_defoc.H",lag_n,1,slow_conv_defoc)
!    !}
!    end do

!---------------------------------------------------------------------------
  call system_clock(timer1)
  timer_sum = 1.*(timer1-timer0)/rate
  call to_history("timer sum (min)", timer_sum/60.)
  write(0,*) "timer sum (min)", timer_sum/60.
!---------------------------------------------------------------------------

end program
