module focus1D_mod

  use sep
  use mem_mod
  use adj_mod

  implicit none
  logical, private, parameter   :: T = .true., F = .false.
  real, private, parameter   :: PI= 3.14159265358979323844

  logical, private              :: verb
  logical, private              :: use_focus_oper,focus_complement,shrink
  logical, private              :: focus_strength_fix,focus_oper_fix
  integer, private              :: lag_n,zero_lag,slow_phys_lag,interp_n
  integer, private              :: focus_oper_n,focus_oper_i=1
  real, private                 :: lag_o,lag_d,focus_strength
  real, private                 :: focus_oper_o,focus_oper_d
  real, private                 :: shrink_factor

  real, dimension(:), allocatable, private           :: weight
  real, dimension(:,:), allocatable, private         :: focus_oper
  real, dimension(:,:), allocatable, private         :: interp

  contains

  subroutine focus1D_init(verb_in, &
         use_focus_oper_in,focus_complement_in,shrink_in, &
         lag_n_in,slow_phys_lag_in,zero_lag_in,lag_o_in,lag_d_in, &
         focus_oper_n_in,focus_oper_o_in,focus_oper_d_in, &
         interp_n_in,focus_strength_in)
!         focus_strength_in,interp_n_in)

    optional                            :: focus_strength_in,interp_n_in
    logical verb_in,use_focus_oper_in,focus_complement_in,shrink_in
    integer lag_n_in,zero_lag_in,slow_phys_lag_in,interp_n_in,focus_oper_n_in
    real lag_o_in,lag_d_in,focus_strength_in,focus_oper_o_in,focus_oper_d_in

    integer interp_mid,i_interp_sinc
    integer i_lag,i_new_lag,i_interp,i_lag_sinc
    real new_lag,d_new_lag,d_sinc,x_sinc

    verb=verb_in
    use_focus_oper=use_focus_oper_in
    focus_complement=focus_complement_in
    shrink=shrink_in
    lag_n=lag_n_in
    lag_o=lag_o_in
    lag_d=lag_d_in
    slow_phys_lag=slow_phys_lag_in
    zero_lag=zero_lag_in
    focus_oper_n=focus_oper_n_in
    focus_oper_o=focus_oper_o_in
    focus_oper_d=focus_oper_d_in
    if(focus_oper_n .eq. 1) then
       focus_oper_fix=T
     else
       focus_oper_fix=F
    end if
    if(present(focus_strength_in)) then
      focus_strength_fix=T
      focus_strength=focus_strength_in
      if((focus_strength < 0.).or.(focus_strength >1.)) then
        write(0,*)'From par: focus_strength=',focus_strength
        call seperr("focus_strength should be >=0 and <=1")
      end if
    else
      focus_strength_fix=F
    end if
    if(present(interp_n_in)) interp_n=interp_n_in
!    if((focus_oper_n .ne. 1).and.(.not. use_focus_oper )) then
!      call seperr("focus_oper_n > 1 implemented only with use_focus_oper")
!    end if

    call mem_alloc(weight, lag_n-1)
    if (shrink) then
      if(mod(interp_n,2) .ne. 1) call seperr("interp_n must be odd")
      interp_mid=(interp_n-1)/2+1
      call mem_alloc(interp,interp_n,slow_phys_lag-1)
      interp=0
      shrink_factor=1./(1.-focus_strength)

      if(verb) write(0,*)'interp_mid=',interp_mid
      if(verb) write(0,*)'shape(interp)=',shape(interp)
      if(verb) write(0,*)'zero_lag=',zero_lag
      if(verb) write(0,*)'shrink_factor=',shrink_factor

      do i_lag=zero_lag+1,slow_phys_lag-1
        if(shrink_factor .eq. 1.) then
          if(i_lag < zero_lag) then
            new_lag=(i_lag-zero_lag)-1
          else
            new_lag=(i_lag-zero_lag)+1
          end if
      if(verb) write(0,*)'shrink_factor is equal to 1!'
        else
          new_lag=(i_lag-zero_lag)*shrink_factor
        end if
        i_new_lag=int(new_lag)
        d_new_lag=abs(new_lag-i_new_lag)
        i_new_lag=zero_lag+i_new_lag
        d_sinc=PI
        do i_interp=1,interp_n
          i_interp_sinc=i_interp-interp_mid
          i_lag_sinc=i_lag+i_interp_sinc
          if((i_lag_sinc >= 1) .and. (i_lag_sinc <= slow_phys_lag-1)) then
            x_sinc=(-float(i_interp_sinc)+d_new_lag)*d_sinc
            if(abs(x_sinc) >  10*tiny(x_sinc)) then
              interp(i_interp,i_lag)=sin(x_sinc)/x_sinc
            else
              interp(i_interp,i_lag)=1.
            end if
          else
            interp(i_interp,i_lag_sinc)=0.
          end if
        end do
        interp(:,zero_lag-(i_lag-zero_lag))=interp(:,i_lag)
      end do
      interp(:,zero_lag)=0.
      interp(interp_mid,zero_lag)=1.
! call snap("Interp.H",interp_n,slow_phys_lag-1,interp)
     weight=0.
    else
     weight=1.
    end if
    if(use_focus_oper) then
      call mem_alloc(focus_oper, lag_n,focus_oper_n)
      call sep_read(focus_oper, "focus_oper")
    end if
  end subroutine

  integer function focus1D_oper_update(focus_oper_i_in)
    integer                :: focus_oper_i_in
    call focus1D_oper_update2(focus_oper_i_in)
    focus1D_oper_update = 0
  end function

  subroutine focus1D_oper_update2(focus_oper_i_in)
    integer                :: focus_oper_i_in
    if(focus_oper_fix) then
      focus_oper_i = 1
    else
      focus_oper_i = focus_oper_i_in
      if(.not. focus_strength_fix) then
        focus_strength=focus_oper_o+(focus_oper_i_in-1)*focus_oper_d
        if(verb) write(0,*)'focus_strength=',focus_strength
      end if
    end if
  end subroutine

  integer function focus1D_op(adj, add, model, data)
    logical             :: adj, add
    real, dimension(:)  :: model, data
    call focus1D_op2(adj, add, model, data)
    focus1D_op = 0
  end function

  subroutine focus1D_op2(adj, add, model, data) 
    logical             :: adj, add
    real, dimension(:)  :: model, data

    integer i_lag,i_new_lag,i_interp
    real new_lag,d_new_lag

    call adjnull (adj,add,model,data )

    if(adj) then
    !{ if (adj)
      if (shrink) then
        do i_lag=1,slow_phys_lag-1
          if(shrink_factor .eq. 1.) then
            if(i_lag < zero_lag) then
              new_lag=(i_lag-zero_lag)-1
            else
              new_lag=(i_lag-zero_lag)+1
            end if
          else
            new_lag=(i_lag-zero_lag)*shrink_factor
          end if
          i_new_lag=int(new_lag)
          i_new_lag=zero_lag+i_new_lag
          if((i_new_lag >=1 ).and.(i_new_lag <= slow_phys_lag-1)) then
            do i_interp=1,interp_n
              model(i_new_lag)= model(i_new_lag) + data(i_lag)*interp(i_interp,i_lag)
            end do
          end if
        end do
        model(slow_phys_lag)= model(slow_phys_lag)+data(slow_phys_lag)
      else
        if(use_focus_oper) then
          model= model + data*focus_oper(:,focus_oper_i)
        else
          if(add) call seperr("focus_1D not implemeted with add=y")
          model(1:zero_lag-2)= focus_strength*data(2:zero_lag-1)
          model(zero_lag-1)= 0.
          model(zero_lag)=focus_strength*data(zero_lag)
          model(zero_lag+1)= 0.
          model(zero_lag+2:slow_phys_lag-1)= &
            focus_strength* data(zero_lag+1:slow_phys_lag-2)
          model(slow_phys_lag)=focus_strength*data(slow_phys_lag)
        end if
      end if
      if(focus_complement) model(1:slow_phys_lag-1)= &
          data(1:slow_phys_lag-1)-model(1:slow_phys_lag-1)
    !} if (adj)
    else
    !{ else ( not adj)
      if (shrink) then
        do i_lag=1,slow_phys_lag-1
          if(shrink_factor .eq. 1.) then
            if(i_lag < zero_lag) then
              new_lag=(i_lag-zero_lag)-1
            else
              new_lag=(i_lag-zero_lag)+1
            end if
          else
            new_lag=(i_lag-zero_lag)*shrink_factor
          end if
          i_new_lag=int(new_lag)
          i_new_lag=zero_lag+i_new_lag
          if((i_new_lag >=1 ).and.(i_new_lag <= slow_phys_lag-1)) then
            do i_interp=1,interp_n
              data(i_lag)= data(i_lag) + model(i_new_lag)*interp(i_interp,i_lag)
            end do
          end if
        end do
        data(slow_phys_lag)= data(slow_phys_lag)+ model(slow_phys_lag)
      else
        if(use_focus_oper) then
          if(verb) write(0,*)'focus_oper_i=',focus_oper_i
!write(0,*)'shape(data)=',shape(data)
!write(0,*)'shape(model)=',shape(model)
!write(0,*)'shape(focus_oper)=',shape(focus_oper)
          data= data + model*focus_oper(:,focus_oper_i)
        else
          if(add) call seperr("focus_1D not implemeted with add=y")
          data(1)= 0.
          data(2:zero_lag-1)= focus_strength*model(1:zero_lag-2)
          data(zero_lag)=focus_strength*model(zero_lag)
          data(zero_lag+1:slow_phys_lag-2)=  &
            focus_strength*model(zero_lag+2:slow_phys_lag-1)
          data(slow_phys_lag)=focus_strength*model(slow_phys_lag)
        end if
      end if
!      data=data*weight
      if(focus_complement) data(1:slow_phys_lag-1)= &
          model(1:slow_phys_lag-1)-data(1:slow_phys_lag-1)
    !} else ( not adj)
    end if
  end subroutine

end module
