module mem_mod

  implicit none
  integer,private                        :: gb0=0, gb1=0
  double precision,private               :: memsize=0.d0

  interface mem_alloc
    module procedure mem_alloc_int_1d
    module procedure mem_alloc_int_2d
    module procedure mem_alloc_int_3d
    module procedure mem_alloc_int_4d
    module procedure mem_alloc_int_5d
    module procedure mem_alloc_int_6d
    module procedure mem_alloc_int_7d
    module procedure mem_alloc_real_1d
    module procedure mem_alloc_real_2d
    module procedure mem_alloc_real_3d
    module procedure mem_alloc_real_4d
    module procedure mem_alloc_real_5d
    module procedure mem_alloc_real_6d
    module procedure mem_alloc_real_7d
    module procedure mem_alloc_complex_1d
    module procedure mem_alloc_complex_2d
    module procedure mem_alloc_complex_3d
    module procedure mem_alloc_complex_4d
    module procedure mem_alloc_complex_5d
    module procedure mem_alloc_complex_6d
    module procedure mem_alloc_complex_7d
  end interface

  interface mem_dealloc
    module procedure mem_dealloc_int_1d
    module procedure mem_dealloc_int_2d
    module procedure mem_dealloc_int_3d
    module procedure mem_dealloc_int_4d
    module procedure mem_dealloc_int_5d
    module procedure mem_dealloc_int_6d
    module procedure mem_dealloc_int_7d
    module procedure mem_dealloc_real_1d
    module procedure mem_dealloc_real_2d
    module procedure mem_dealloc_real_3d
    module procedure mem_dealloc_real_4d
    module procedure mem_dealloc_real_5d
    module procedure mem_dealloc_real_6d
    module procedure mem_dealloc_real_7d
    module procedure mem_dealloc_complex_1d
    module procedure mem_dealloc_complex_2d
    module procedure mem_dealloc_complex_3d
    module procedure mem_dealloc_complex_4d
    module procedure mem_dealloc_complex_5d
    module procedure mem_dealloc_complex_6d
    module procedure mem_dealloc_complex_7d
  end interface

  contains

!!---------------------------------------------------------- 

  subroutine mem_print()
    gb1 = int(memsize/1024./1024./1024.)
    if(gb1 > gb0 .or. gb1 < gb0) write(0,*) "Total CPU memory allocated in GB", real(memsize/1024./1024./1024.)
    gb0 = gb1
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_add_4(n1)
    integer                               :: n1
    memsize = memsize + 1.d0*4.*n1
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_remove_4(n1)
    integer                               :: n1
    memsize = memsize - 1.d0*4.*n1
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_add_8(n1)
    integer                               :: n1
    memsize = memsize + 1.d0*8.*n1
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_remove_8(n1)
    integer                               :: n1
    memsize = memsize - 1.d0*8.*n1
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_int_1d(array, n1)
    integer                               :: n1
    integer, dimension(:), allocatable       :: array
    if(.not. allocated(array)) then
      allocate(array(n1))
      memsize = memsize + 1.d0*4.*n1
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_int_2d(array, n1, n2)
    integer                            :: n1, n2
    integer, dimension(:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2))
      memsize = memsize + 1.d0*4.*n1*n2
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_int_3d(array, n1, n2, n3)
    integer                              :: n1, n2, n3
    integer, dimension(:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3))
      memsize = memsize + 1.d0*4.*n1*n2*n3
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_int_4d(array, n1, n2, n3, n4)
    integer                                :: n1, n2, n3, n4
    integer, dimension(:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4))
      memsize = memsize + 1.d0*4.*n1*n2*n3*n4
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_int_5d(array, n1, n2, n3, n4, n5)
    integer                                  :: n1, n2, n3, n4, n5
    integer, dimension(:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5))
      memsize = memsize + 1.d0*4.*n1*n2*n3*n4*n5
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_int_6d(array, n1, n2, n3, n4, n5, n6)
    integer                                    :: n1, n2, n3, n4, n5, n6
    integer, dimension(:,:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5,n6))
      memsize = memsize + 1.d0*4.*n1*n2*n3*n4*n5*n6
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_int_7d(array, n1, n2, n3, n4, n5, n6, n7)
    integer                                      :: n1, n2, n3, n4, n5, n6, n7
    integer, dimension(:,:,:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5,n6,n7))
      memsize = memsize + 1.d0*4.*n1*n2*n3*n4*n5*n6*n7
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_real_1d(array, n1)
    integer                               :: n1
    real, dimension(:), allocatable       :: array
    if(.not. allocated(array)) then
      allocate(array(n1))
      memsize = memsize + 1.d0*4.*n1
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_real_2d(array, n1, n2)
    integer                            :: n1, n2
    real, dimension(:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2))
      memsize = memsize + 1.d0*4.*n1*n2
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_real_3d(array, n1, n2, n3)
    integer                              :: n1, n2, n3
    real, dimension(:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3))
      memsize = memsize + 1.d0*4.*n1*n2*n3
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_real_4d(array, n1, n2, n3, n4)
    integer                                :: n1, n2, n3, n4
    real, dimension(:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4))
      memsize = memsize + 1.d0*4.*n1*n2*n3*n4
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_real_5d(array, n1, n2, n3, n4, n5)
    integer                                  :: n1, n2, n3, n4, n5
    real, dimension(:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5))
      memsize = memsize + 1.d0*4.*n1*n2*n3*n4*n5
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_real_6d(array, n1, n2, n3, n4, n5, n6)
    integer                                    :: n1, n2, n3, n4, n5, n6
    real, dimension(:,:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5,n6))
      memsize = memsize + 1.d0*4.*n1*n2*n3*n4*n5*n6
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_real_7d(array, n1, n2, n3, n4, n5, n6, n7)
    integer                                      :: n1, n2, n3, n4, n5, n6, n7
    real, dimension(:,:,:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5,n6,n7))
      memsize = memsize + 1.d0*4.*n1*n2*n3*n4*n5*n6*n7
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_complex_1d(array, n1)
    integer                               :: n1
    complex, dimension(:), allocatable       :: array
    if(.not. allocated(array)) then
      allocate(array(n1))
      memsize = memsize + 1.d0*8.*n1
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_complex_2d(array, n1, n2)
    integer                            :: n1, n2
    complex, dimension(:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2))
      memsize = memsize + 1.d0*8.*n1*n2
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_complex_3d(array, n1, n2, n3)
    integer                              :: n1, n2, n3
    complex, dimension(:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3))
      memsize = memsize + 1.d0*8.*n1*n2*n3
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_complex_4d(array, n1, n2, n3, n4)
    integer                                :: n1, n2, n3, n4
    complex, dimension(:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4))
      memsize = memsize + 1.d0*8.*n1*n2*n3*n4
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_complex_5d(array, n1, n2, n3, n4, n5)
    integer                                  :: n1, n2, n3, n4, n5
    complex, dimension(:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5))
      memsize = memsize + 1.d0*8.*n1*n2*n3*n4*n5
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_complex_6d(array, n1, n2, n3, n4, n5, n6)
    integer                                    :: n1, n2, n3, n4, n5, n6
    complex, dimension(:,:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5,n6))
      memsize = memsize + 1.d0*8.*n1*n2*n3*n4*n5*n6
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_alloc_complex_7d(array, n1, n2, n3, n4, n5, n6, n7)
    integer                                      :: n1, n2, n3, n4, n5, n6, n7
    complex, dimension(:,:,:,:,:,:,:), allocatable  :: array
    if(.not. allocated(array)) then
      allocate(array(n1,n2,n3,n4,n5,n6,n7))
      memsize = memsize + 1.d0*8.*n1*n2*n3*n4*n5*n6*n7
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_int_1d(array)
    integer, dimension(:), allocatable       :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_int_2d(array)
    integer, dimension(:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_int_3d(array)
    integer, dimension(:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_int_4d(array)
    integer, dimension(:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_int_5d(array)
    integer, dimension(:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_int_6d(array)
    integer, dimension(:,:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_int_7d(array)
    integer, dimension(:,:,:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_real_1d(array)
    real, dimension(:), allocatable       :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_real_2d(array)
    real, dimension(:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_real_3d(array)
    real, dimension(:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_real_4d(array)
    real, dimension(:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_real_5d(array)
    real, dimension(:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_real_6d(array)
    real, dimension(:,:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_real_7d(array)
    real, dimension(:,:,:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*4.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_complex_1d(array)
    complex, dimension(:), allocatable       :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*8.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_complex_2d(array)
    complex, dimension(:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*8.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_complex_3d(array)
    complex, dimension(:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*8.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_complex_4d(array)
    complex, dimension(:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*8.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_complex_5d(array)
    complex, dimension(:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*8.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_complex_6d(array)
    complex, dimension(:,:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*8.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

  subroutine mem_dealloc_complex_7d(array)
    complex, dimension(:,:,:,:,:,:,:), allocatable  :: array
    if(allocated(array)) then
      deallocate(array)
      memsize = memsize - 1.d0*8.*size(array)
      call mem_print()
    end if
  end subroutine

!!---------------------------------------------------------- 

end module
