module mysample_mod
! Sampling stepper
! Last edited by Joe Jennings 07/24/2016

  use stepper_obj_mod
  use prob_obj_mod

  implicit none

  private
  public :: mysample

  type, extends(stepper_obj)                    :: mysample ! extend line search to back tracking method
  !Variable extension
    integer                                     :: n = 3
    real                                        :: cmin = 0.5
    real                                        :: cmax = 1.5
    double precision                            :: ntry = 10
    double precision                            :: alpha = 0.
    double precision                            :: r = 0.25
    real, dimension(:), allocatable             :: modl
    real, dimension(:), allocatable             :: res
    real, dimension(:), allocatable             :: dres
    real, dimension(:), allocatable             :: c
    double precision, dimension(:), allocatable :: f
  contains
    procedure                                   :: run
  end type

  interface mysample
    module procedure const
  end interface

  contains

  function const(n,cmin,cmax,ntry,alpha,r) result(this)
    optional                       :: n, cmin, cmax, ntry, alpha, r
    integer                    :: n
    real                       :: cmin
    real                       :: cmax
    integer                    :: ntry
    real                       :: alpha
    real                       :: r
    type(mysample)             :: this
    integer                    :: i
    if(present(n)) this%n = n 
    if(present(cmin)) this%cmin = cmin
    if(present(cmax)) this%cmax = cmax
    if(present(ntry)) this%ntry = ntry
    if(present(alpha)) this%alpha = alpha
    if(present(r)) this%r = r 
    allocate(this%c(this%n))
    allocate(this%f(this%n))
    do i=1,this%n
      this%c(i) = (i-1.)*(this%cmax-this%cmin)/(this%n-1.) + this%cmin
    end do
  end function

  !Calling function to choose and run a solver
  integer function run(this, modl, prob, dmodl, alpha, success)
    class(mysample)             :: this
    real,dimension(:)                 :: modl            ! model (input/output)
    class(prob_obj)                   :: prob
    real,dimension(:)                 :: dmodl           ! search direction (input)
    double precision                  :: alpha           ! step size (output)
    logical                           :: success
    integer                           :: stat
    double precision                  :: f0
    integer                           :: i
    integer                           :: itry
    integer                           :: test

    if(.not. allocated(this%modl)) allocate(this%modl(prob%nmodl))
    if(.not. allocated(this%res)) allocate(this%res(prob%ndata))
    if(.not. allocated(this%dres)) allocate(this%dres(prob%ndata))
    success = .false.
    stat = prob%get_obj(modl, f0)
    alpha = this%alpha

    ! run the stepper
    do itry=1,2*this%ntry
      if(alpha .eq. 0. .or. itry .eq. this%ntry) then
        stat = prob%get_dres(modl, dmodl, this%dres)
        stat = prob%get_res(modl, this%res)
        alpha = - sum(dprod(this%dres, this%res)) / sum(dprod(this%dres, this%dres))
      end if
      do i=1,this%n
        this%modl = modl + this%c(i)*alpha*dmodl
        stat = prob%get_obj(this%modl, this%f(i))
      end do
      if(minval(this%f) < f0) then
        alpha = alpha * this%c(sum(minloc(this%f)))
        success = .true.
        exit
      else
        alpha = alpha * this%r
      end if
    end do
    if(success) then
      ! finish line search, update model
      this%alpha = alpha
      modl = modl + alpha*dmodl
    end if
    run = 0
  end function

end module
