module prob_l_l2_reg_mod
! Linear L2 regularization term
! Last edited by Ali Almomin 08/24/2015
! 0 = L m - d

  use prob_obj_mod

  implicit none

  private
  public :: prob_l_l2_reg

  logical, parameter                  :: T = .true., F = .false.

  type, extends(prob_obj)             :: prob_l_l2_reg
    private
    real, dimension(:), pointer       :: data
    procedure(oper), nopass, pointer  :: lop
  contains
    private
    procedure                         :: objf
    procedure                         :: resf
    procedure                         :: gradf
    procedure                         :: dresf
  end type

  interface prob_l_l2_reg
    module procedure const
  end interface

  interface
    integer function oper(adj, add, modl, data)
      real,dimension(:) :: modl,data
      logical           :: adj,add
    end function
  end interface

  contains

  type(prob_l_l2_reg) function const(lop, nmodl, ndata)
    procedure(oper)              :: lop
    integer                      :: nmodl           ! size of modl (input)
    integer                      :: ndata           ! size of data (input)
    integer                      :: stat           ! 
    const%lop => lop
    const%nmodl = nmodl
    const%ndata = ndata
    stat = const%alloc()
  end function

  integer function resf(this, modl, res)      ! function for calculating the residual
    class(prob_l_l2_reg)        :: this
    real, dimension(:) :: modl            ! model (input)
    real, dimension(:) :: res             ! residual (output)
    integer            :: stat            ! status flag
    stat = this%lop(F, F, modl, res)
    resf = 0
  end function

  integer function objf(this, res, obj)        ! function for calculating objective function
    class(prob_l_l2_reg)        :: this
    real, dimension(:) :: res             ! residual (input)
    double precision   :: obj             ! objective function value (output)
    obj = 0.5*(sum(dprod(res,res)))
    objf = 0
  end function

  integer function gradf(this, modl, res, grad) ! function for calculating gradient
    class(prob_l_l2_reg)        :: this
    real, dimension(:) :: modl            ! model (input)
    real, dimension(:) :: res             ! residual (input)
    real, dimension(:) :: grad            ! gradient (output)
    integer            :: stat            ! status flag
    stat = this%lop(T, F, grad, res)
    gradf = 0
  end function

  integer function dresf(this, modl, dmodl, dres) ! function for calculating gradient in data space
    class(prob_l_l2_reg)        :: this
    real, dimension(:) :: modl           ! model (input)
    real, dimension(:) :: dmodl          ! gradient (input)
    real, dimension(:) :: dres           ! gradient in data space (output)
    integer            :: stat           ! status flag
    stat = this%lop(F, F, dmodl, dres)
    dresf = 0
  end function

end module
