module prob_l_l2_reg_data_mod
! Linear L2 regularization term
! Last edited by Ali Almomin 08/24/2015
! 0 = L m - d

  use prob_obj_mod

  implicit none

  private
  public :: prob_l_l2_reg_data

  logical, parameter                  :: T = .true., F = .false.

  type, extends(prob_obj)             :: prob_l_l2_reg_data
    private
    real, dimension(:), pointer        :: data
    real, dimension(:), allocatable    :: tmp_modl1
    real, dimension(:), allocatable    :: tmp_modl2
    procedure(oper),     nopass, pointer   :: fop
    procedure(oper),     nopass, pointer   :: nlopext
    procedure(oper),     nopass, pointer   :: lopext
    procedure(update),   nopass, pointer :: uoplextbk
    procedure(update),   nopass, pointer :: uopnlext
    procedure(extupdate), nopass, pointer :: uoplext
  contains
    private
    procedure                         :: objf
    procedure                         :: resf
    procedure                         :: gradf
    procedure                         :: dresf
  end type

  interface prob_l_l2_reg_data
    module procedure const
  end interface

  interface
    integer function oper(adj, add, modl, data)
      real,dimension(:) :: modl,data
      logical           :: adj,add
    end function
    integer function update(modl)
      real :: modl
    end function
    integer function extupdate(modl)
      real,dimension(:) :: modl
    end function
  end interface

  contains

  type(prob_l_l2_reg_data) function const(fop, nlopext, lopext, uopnlext, &
                                     uoplextbk, uoplext, nmodl, ndata)
    procedure(oper)              :: fop
    procedure(oper)              :: nlopext
    procedure(oper)              :: lopext
    procedure(update)            :: uopnlext
    procedure(update)            :: uoplextbk
    procedure(extupdate)         :: uoplext
    integer                      :: nmodl           ! size of modl (input)
    integer                      :: ndata           ! size of data (input)
    integer                      :: stat           ! 
    const%fop       => fop
    const%nlopext   => nlopext
    const%lopext    => lopext
    const%uopnlext  => uopnlext
    const%uoplext   => uoplext
    const%uoplextbk => uoplextbk
    const%nmodl = nmodl
    const%ndata = ndata
    allocate(const%tmp_modl1(const%nmodl), const%tmp_modl2(const%nmodl))
    stat = const%alloc()
  end function

  integer function resf(this, modl, res)      ! function for calculating the residual
    class(prob_l_l2_reg_data)        :: this
    real, dimension(:) :: modl            ! model (input)
    real, dimension(:) :: res             ! residual (output)
    integer            :: stat            ! status flag
    stat = this%uopnlext(modl(this%nmodl))
    stat = this%fop(F, F, modl, this%tmp_modl1)
    stat = this%nlopext(F, F, this%tmp_modl1, res)
    resf = 0
  end function

  integer function objf(this, res, obj)        ! function for calculating objective function
    class(prob_l_l2_reg_data)        :: this
    real, dimension(:) :: res             ! residual (input)
    double precision   :: obj             ! objective function value (output)
    obj = 0.5*(sum(dprod(res,res)))
    objf = 0
  end function

  integer function gradf(this, modl, res, grad) ! function for calculating gradient
    class(prob_l_l2_reg_data)        :: this
    real, dimension(:) :: modl            ! model (input)
    real, dimension(:) :: res             ! residual (input)
    real, dimension(:) :: grad            ! gradient (output)
    integer            :: stat            ! status flag
    ! Update physical slowness
    stat = this%uopnlext (modl(this%nmodl))
    stat = this%uoplextbk(modl(this%nmodl))
    ! Update extended with a more focused c
    stat = this%fop(F, F, modl, this%tmp_modl1)
    stat = this%uoplext(this%tmp_modl1(1:this%nmodl - 1))
    ! Compute gradient in s
    stat = this%lopext(T, F, grad, res)
    ! gradient in c
    stat = this%nlopext(T, F, this%tmp_modl2, res)
    stat = this%fop(T, F, grad, this%tmp_modl2)
    gradf = 0
  end function

  integer function dresf(this, modl, dmodl, dres) ! function for calculating gradient in data space
    class(prob_l_l2_reg_data)        :: this
    real, dimension(:) :: modl           ! model (input)
    real, dimension(:) :: dmodl          ! gradient (input)
    real, dimension(:) :: dres           ! gradient in data space (output)
    integer            :: stat           ! status flag
    ! Update physical slowness
    stat = this%uopnlext (modl(this%nmodl))
    stat = this%uoplextbk(modl(this%nmodl))
    ! Update with a more focused c
    stat = this%fop(F, F, modl, this%tmp_modl1)
    stat = this%uoplext(this%tmp_modl1(1:this%nmodl - 1))
    ! Compute data space gradient s
    stat = this%lopext(F, F, dmodl, dres)
    ! Data space gradient in c
    stat = this%fop(F, F, dmodl, this%tmp_modl2)
    stat = this%nlopext(F, F, this%tmp_modl2, dres)
    dresf = 0
  end function

end module
