module prob_nl_l2_diff_onlyc_mod
! Nonlinear L2 difference objective form
! for 1D simple EFWI
! Last edited by Ali Almomin 08/24/2015
! 0 = Wd (L(S Wm m) - d)

  use prob_obj_mod

  implicit none

  private
  public :: prob_nl_l2_diff_onlyc

  logical, parameter                           :: T = .true., F = .false.

  type, extends(prob_obj)                       :: prob_nl_l2_diff_onlyc
    private
    real, dimension(:), pointer        :: data
    real, dimension(:), allocatable    :: tmp_data
    procedure(oper), nopass, pointer   :: nlop
    procedure(oper), nopass, pointer   :: lop
    procedure(oper), nopass, pointer   :: lopext
    procedure(oper), nopass, pointer   :: nlopext
    procedure(update), nopass, pointer :: uopl
    procedure(update), nopass, pointer :: uoplext
    procedure(update), nopass, pointer :: uopnlext
  contains
    procedure                                  :: objf
    procedure                                  :: resf
    procedure                                  :: gradf
    procedure                                  :: dresf
  end type

  interface prob_nl_l2_diff_onlyc
    module procedure const
  end interface

  interface
    integer function oper(adj, add, modl, data)
      real,dimension(:) :: modl,data
      logical           :: adj,add
    end function
    integer function update(modl)
      real :: modl
    end function
  end interface

  contains

  type(prob_nl_l2_diff_onlyc) function const(nlop, nlopext, nmodl, ndata, data)
    procedure(oper)              :: nlop, nlopext, lop, lopext
    procedure(update)            :: uopl, uoplext, uopnlext
    integer                      :: nmodl          ! size of modl (input)
    integer                      :: ndata          ! size of data (input)
    integer                      :: stat           ! 
    real, dimension(:), target   :: data           ! data (input)
    const%nlop     => nlop
    const%nlopext  => nlopext
    const%data     => data
    const%nmodl    = nmodl
    const%ndata    = ndata
    allocate(const%tmp_data(const%ndata))
    stat = const%alloc()
  end function

  integer function objf(this, res, obj)   ! function for calculating objective function
    class(prob_nl_l2_diff_onlyc)   :: this
    real, dimension(:)       :: res  ! residual (input)
    double precision         :: obj  ! objective function value (output)
    obj = 0.5*(sum(dprod(res,res)))
    objf = 0
  end function

  integer function resf(this, modl, res)  ! function for calculating the residual
    class(prob_nl_l2_diff_onlyc) :: this
    real, dimension(:)     :: modl  ! model (input)
    real, dimension(:)     :: res   ! residual (output)
    integer                :: stat  ! status flag
    stat = this%nlop(F, F, modl, this%tmp_data)
    stat = this%nlopext(F, F, modl, res)
    res = res + this%tmp_data
    res = res - this%data
    resf = 0
  end function

  integer function gradf(this, modl, res, grad) ! function for calculating gradient
    class(prob_nl_l2_diff_onlyc)        :: this
    real, dimension(:) :: modl            ! model (input)
    real, dimension(:) :: res             ! residual (input)
    real, dimension(:) :: grad            ! gradient (output)
    integer            :: stat            ! status flag
    ! Compute gradient in c
    stat = this%nlopext(T, F, grad, res)
    gradf = 0
  end function

  integer function dresf(this, modl, dmodl, dres) ! function for calculating gradient in data space
    class(prob_nl_l2_diff_onlyc)        :: this
    real, dimension(:) :: modl           ! model (input)
    real, dimension(:) :: dmodl          ! gradient (input)
    real, dimension(:) :: dres           ! gradient in data space (output)
    integer            :: stat           ! status flag
    ! Conjugate gradient in c
    stat = this%nlopext(F, F, dmodl, dres)
    dresf = 0
  end function

end module
