!! Non-linear modeling operator
!! <p>
!! Implementation of script-L in notes
!!
!! @author Biondo Biondi
!! @version 2016.07.13
module prop1Daxis_mod

  use mem_mod
  use adj_mod
  use axis_mod
  use lint1_stat

  implicit none

  logical, private, parameter                   :: T = .true., F = .false.

  type(axis)                                    :: daxis, waxis   !! temporal axes
  real, private                                 :: dist           !! propagation distance
  real, private                                 :: wavelet_tdelay !! intial wavelet delay
  real, private                                 :: slow_phys      !! physical slowness
  real, private                                 :: dly

  real, dimension(:), allocatable, private      :: wavelet             !! input wavelet

  contains

  subroutine prop1Daxis_init(dax_in, wax_in, dly_in, dist_in)
    type(axis)         :: dax_in, wax_in
    real               :: dly_in
    real               :: dist_in

    daxis         = dax_in
    waxis         = wax_in
    dly           = dly_in
    dist          = dist_in

    if (daxis%d .ne. waxis%d) then
      write(0,*)'dtdata=',daxis%d
      write(0,*)'dtwavlet=',waxis%d
      call seperr("dtdata must be equal to dtwavelet")
    end if

    call mem_alloc(wavelet,waxis%n)
  end subroutine

  !! Updates the slowness
  integer function prop1Daxis_slow_update(slow_phys_in)
    real                :: slow_phys_in
    call prop1Daxis_slow_update2(slow_phys_in)
    prop1Daxis_slow_update = 0
  end function

  !! Updates the slowness
  subroutine prop1Daxis_slow_update2(slow_phys_in)
    real                :: slow_phys_in
    slow_phys = slow_phys_in
  end subroutine

  !! Updates the wavelet
  integer function prop1Daxis_wavelet_update(wavelet_in)
    real, dimension(:)  :: wavelet_in !! new wavelet
    call prop1Daxis_wavelet_update2(wavelet_in)
    prop1Daxis_wavelet_update = 0
  end function

  !! Updates the wavelet
  subroutine prop1Daxis_wavelet_update2(wavelet_in)
    real, dimension(:)  :: wavelet_in !! new wavelet
    wavelet = wavelet_in
  end subroutine

  !! Applies the linear prop1D operator
  integer function prop1Daxis_op(adj, add, model, data)
    logical           :: adj   !! apply forward or adjoint
    logical           :: add   !! add or clear input
    real,dimension(:) :: model !! input wavelet
    real,dimension(:) :: data  !! shifted wavelet
    call prop1Daxis_op2(adj, add, model, data)
    prop1Daxis_op = 0
  end function

  !! Applies the linear prop1D operator
  subroutine prop1Daxis_op2(adj, add, model, data)
    logical           :: adj   !! apply forward or adjoint
    logical           :: add   !! add or clear input
    real,dimension(:) :: model !! input wavelet
    real,dimension(:) :: data  !! shifted wavelet

    real               :: time_prop

    call adjnull (adj,add,model,data )

    if(adj) then
      time_prop = (dist * slow_phys) - (dly + daxis%o)
      call lint1_stat_init (waxis%o, daxis%d, -time_prop)
      call lint1_stat_lop2 (T,add, model, data)
    else
      time_prop = (dist * slow_phys) - (dly + daxis%o)
      call lint1_stat_init (waxis%o, daxis%d, -time_prop)
      call lint1_stat_lop2 (F,add, model, data)
    end if
  end subroutine

  !! Applies prop1D non-linear operator
  integer function prop1Daxis_nlop(adj, add, model ,data)
    logical           :: adj   !! apply forward or adjoint
    logical           :: add   !! add or clear input
    real,dimension(:) :: model !! input wavelet
    real,dimension(:) :: data  !! shifted wavelet
    call prop1Daxis_nlop2(adj,add,model,data)
    prop1Daxis_nlop = 0
  end function

  !! Applies prop1D non-linear operator
  subroutine prop1Daxis_nlop2(adj, add, model, data)
    logical           :: adj   !! apply forward or adjoint
    logical           :: add   !! add or clear input
    real,dimension(:) :: model !! input wavelet
    real,dimension(:) :: data  !! shifted wavelet

    real              :: time_prop

    if(adj) then
      call seperr("Adjoint of prop1Daxis_nlop has not been implemented.")
    else
      time_prop=(dist * model( size(model) ))-(dly + daxis%o) ! assumes phys slow is last sample
      call lint1_stat_init (waxis%o,daxis%d,-time_prop)
      call lint1_stat_lop2 (F,add, wavelet, data)
    endif
  end subroutine

end module
