! implementation of L in notes
module prop_der11D_mod

  use mem_mod
  use adj_mod
  use lint1_stat  
  use igrad1
  use igrad1_cent

  implicit none

  logical, private, parameter                      :: T = .true., F = .false.

  integer, private                           :: t_n,wavelet_n,lag_n
  integer, private                           :: slow_phys_lag
  real, private                              :: lag_o,lag_d,prop_dist
  real, private                              :: wavelet_o,wavelet_d,t_o,t_d
  real, private                              :: wavelet_tdelay
  real, private                              :: slow_phys,dtdslow
  real, private                              :: weight_forw,weight_adj

  real, dimension(:), allocatable, private   :: der1_wavelet,der1_wavelet_shift

  contains

  subroutine prop_der11D_init(t_n_in,wavelet_n_in,lag_n_in,lag_o_in,lag_d_in, &
      wavelet_o_in,wavelet_d_in,t_o_in,t_d_in,wavelet_tdelay_in,prop_dist_in)
    integer                        :: t_n_in,wavelet_n_in,lag_n_in
    real                           :: lag_o_in,lag_d_in,prop_dist_in
    real                           :: wavelet_o_in,wavelet_d_in,t_o_in,t_d_in
    real                           :: wavelet_tdelay_in

    t_n = t_n_in
    wavelet_n = wavelet_n_in
    lag_n = lag_n_in
    lag_o = lag_o_in
    lag_d = lag_d_in
    wavelet_o=wavelet_o_in
    wavelet_d=wavelet_d_in
    t_o=t_o_in
    t_d=t_d_in
    wavelet_tdelay=wavelet_tdelay_in
    prop_dist = prop_dist_in

    slow_phys_lag=lag_n
    dtdslow=-prop_dist
!    weight=1./(float(t_n))
!    weight_adj=1./float(8.2*t_n)
!    weight_adj=1./(7.5*t_n)
    weight_adj=1
    weight_forw=1
!    weight=1.

      write(0,*)'wavelet_n=',wavelet_n
    call mem_alloc(der1_wavelet, wavelet_n)
    call mem_alloc(der1_wavelet_shift, t_n)

    if (wavelet_d .ne. t_d) then
      write(0,*)'t_d=',t_d
      write(0,*)'wavelet_d=',wavelet_d
      call seperr("wavelet_d must be equal to t_d_data")
    end if
  end subroutine

  integer function prop_der11D_phys_slow_update(slow_phys_in)
    real                :: slow_phys_in
    call prop_der11D_phys_slow_update2(slow_phys_in)
    prop_der11D_phys_slow_update = 0
  end function

  subroutine prop_der11D_phys_slow_update2(slow_phys_in)
    real                :: slow_phys_in

    slow_phys = slow_phys_in
  end subroutine

  integer function prop_der11D_der1_wavelet_update(wavelet_in) 
    real, dimension(:)  :: wavelet_in
    call prop_der11D_der1_wavelet_update2(wavelet_in)
    prop_der11D_der1_wavelet_update=0
  end function

  subroutine prop_der11D_der1_wavelet_update2(wavelet_in) ! Copy wavelet
    real, dimension(:)   :: wavelet_in
!    call igrad1_lop2(F,F,wavelet_in,der1_wavelet) 
write(0,*)'shape(wavelet_in)=',shape(wavelet_in)
write(0,*)'shape(der1_wavelet)=',shape(der1_wavelet)
    call igrad1_cent_lop2(F,F,wavelet_in,der1_wavelet) 
    der1_wavelet = der1_wavelet/wavelet_d
  end subroutine

  integer function prop_der11D_op(adj, add, model, data)
    logical            :: adj, add
    real,dimension(:)  :: model, data
    call prop_der11D_op2(adj, add, model, data)
    prop_der11D_op = 0
  end function

  subroutine prop_der11D_op2(adj, add, model, data)
    logical                   :: adj, add
    real, dimension(:) :: model, data
    real                   :: time_prop

    call adjnull (adj,add,model,data )

    if(adj) then
      time_prop=(prop_dist*slow_phys)-(wavelet_tdelay+t_o)
      call lint1_stat_init (wavelet_o,t_d,-time_prop)
      call lint1_stat_lop2 (F,F, der1_wavelet, der1_wavelet_shift)
      model(slow_phys_lag)= model(slow_phys_lag) + &
        dot_product(der1_wavelet_shift,data)*dtdslow*weight_adj
!        dot_product(der1_wavelet_shift,data)*(dtdslow/t_n)
!        sum(der1_wavelet_shift*data)*(dtdslow/t_n)
    else
      time_prop=(prop_dist*slow_phys)-(wavelet_tdelay+t_o)
      call lint1_stat_init (wavelet_o,t_d,-time_prop)
      call lint1_stat_lop2 (F,F, der1_wavelet, der1_wavelet_shift)
!      data=data + der1_wavelet_shift*model(slow_phys_lag)*(dtdslow/t_n)
      data=data + der1_wavelet_shift*model(slow_phys_lag)*dtdslow*weight_forw
    end if
  end subroutine

end module
