! implementation of L~ in notes
module prop_extend1D_mod

  use mem_mod
  use adj_mod
  use tcai1_nb  
  use lint1_stat  

  implicit none

  logical, private, parameter                   :: T = .true., F = .false.

  logical, private                              :: slow_extend
  integer, private                              :: t_n,wavelet_n,lag_n,zero_lag
  integer, private                              :: back_lag_n
  real, private                                 :: lag_o,lag_d,prop_dist
  real, private                                 :: back_lag_o,back_lag_d
  real, private                                 :: wavelet_o,wavelet_d,t_o,t_d
  real, private                                 :: wavelet_tdelay
  real, private                                 :: back_slow_phys,weight

  real, dimension(:), allocatable, private       :: model_conv
  real, dimension(:), allocatable, private       :: wavelet_shift
  real, dimension(:), allocatable, private       :: wavelet
  real, dimension(:), pointer, private           :: wavelet_shift_p

  contains

  subroutine prop_extend1D_init(t_n_in,wavelet_n_in, &
      lag_n_in,lag_o_in,lag_d_in, &
      back_lag_n_in,back_lag_o_in,back_lag_d_in, &
      wavelet_o_in,wavelet_d_in,t_o_in,t_d_in,wavelet_tdelay_in,prop_dist_in)
    integer                        :: t_n_in,wavelet_n_in,lag_n_in,back_lag_n_in
    real                           :: lag_o_in,lag_d_in,prop_dist_in
    real                           :: back_lag_o_in,back_lag_d_in
    real                           :: wavelet_o_in,wavelet_d_in,t_o_in,t_d_in
    real                           :: wavelet_tdelay_in
!    double precision                :: test

    t_n = t_n_in
    wavelet_n = wavelet_n_in
    lag_n = lag_n_in
    lag_o = lag_o_in
    lag_d = lag_d_in
    back_lag_n = back_lag_n_in
    back_lag_o = back_lag_o_in
    back_lag_d = back_lag_d_in
    wavelet_o=wavelet_o_in
    wavelet_d=wavelet_d_in
    t_o=t_o_in
    t_d=t_d_in
    wavelet_tdelay=wavelet_tdelay_in
    prop_dist = prop_dist_in
!    weight=1./sqrt(float(lag_n))
    weight=1.
!    weight=25.
!    weight=1./5

    if (wavelet_d .ne. t_d) then
      write(0,*)'t_d=',t_d
      write(0,*)'wavelet_d=',wavelet_d
      call seperr("wavelet_d must be equal to t_d_data")
    end if

    slow_extend = lag_n >1
    if (slow_extend) then
!      test=mod(dble(-lag_o),dble(lag_d))
!      if (test .ne. 0.) then 
!        write(0,*)'test=',test
!        write(0,*)'lag_o=',dble(lag_o),'lag_d=',dble(lag_d)
      if (mod(lag_o,lag_d) .gt. tiny(lag_o)) then
        write(0,*)'lag_o=',lag_o,'lag_d=',lag_d
        call seperr("lag_o must be a multiple of lag_d")
      end if
      zero_lag=nint(-lag_o/lag_d+1) 
!write(0,*)'zero_lag=',zero_lag
      if ((zero_lag < 2) .or. (zero_lag > lag_n)) &
        call seperr("zero_lag not within lag interval")
    else
      call seperr("Why call prop_extend_1D when slowness is not extended?")
    end if

    if (back_lag_n .ne. 1) then
      call seperr("back_lag_n /= 1 in prop_extend_1D")
    end if

    call mem_alloc(model_conv, lag_n-1)
    call mem_alloc(wavelet, wavelet_n)

    call mem_alloc(wavelet_shift, t_n)
    allocate(wavelet_shift_p(t_n))

  end subroutine

  integer function prop_extend1D_back_slow_update(back_slow_phys_in)
    real                :: back_slow_phys_in
    call prop_extend1D_back_slow_update2(back_slow_phys_in)
    prop_extend1D_back_slow_update = 0
  end function

  subroutine prop_extend1D_back_slow_update2(back_slow_phys_in)
    real                :: back_slow_phys_in
    back_slow_phys = back_slow_phys_in
  end subroutine

  integer function prop_extend1D_wavelet_update(wavelet_in)
    real, dimension(:)  :: wavelet_in
    call prop_extend1D_wavelet_update2(wavelet_in)
    prop_extend1D_wavelet_update = 0
  end function

  subroutine prop_extend1D_wavelet_update2(wavelet_in)
    real, dimension(:)  :: wavelet_in
    wavelet = wavelet_in
  end subroutine

  integer function prop_extend1D_op(adj, add, model, data)
    logical            :: adj, add
    real,dimension(:)  :: model, data
    call prop_extend1D_op2(adj, add, model, data)
    prop_extend1D_op = 0
  end function

  subroutine prop_extend1D_op2(adj, add, model, data)
    logical                   :: adj, add
    real, dimension(:) :: model, data

    real                   :: time_prop

    call adjnull (adj,add,model,data )


    if(adj) then
      time_prop=(prop_dist*back_slow_phys)-(wavelet_tdelay+t_o-lag_o)
      call lint1_stat_init (wavelet_o,t_d,-time_prop)
      call lint1_stat_lop2 (F,F, wavelet, wavelet_shift)
      wavelet_shift_p=wavelet_shift*weight
      call tcai1_nb_init(wavelet_shift_p)
      model_conv(1:lag_n-1)=model(1:lag_n-1)
      call tcai1_nb_lop2(T,add,model_conv,data)
      model(1:lag_n-1)=model_conv(1:lag_n-1)
      model(zero_lag)=0
      model(lag_n)=model(lag_n)
    else
      time_prop=(prop_dist*back_slow_phys)-(wavelet_tdelay+t_o-lag_o)
      call lint1_stat_init (wavelet_o,t_d,-time_prop)
      call lint1_stat_lop2 (F,F, wavelet, wavelet_shift)
      wavelet_shift_p=wavelet_shift*weight
      call tcai1_nb_init(wavelet_shift_p)
      model_conv(1:lag_n-1)=model(1:lag_n-1)
      model_conv(zero_lag)=0
      call tcai1_nb_lop2(F,add,model_conv,data)

    end if
  end subroutine

end module
