! implementation of L~^dot in notes
module prop_extend_der11D_mod

  use mem_mod
  use adj_mod
  use tcai1_nb  
  use lint1_stat  
  use igrad1
  use igrad1_cent

  implicit none

  logical, private, parameter                :: T = .true., F = .false.

  logical, private                           :: back_slow_extend
  integer, private                           :: t_n,wavelet_n,lag_n
  integer, private                           :: back_zero_lag
  integer, private                           :: back_lag_n
  integer, private                           :: slow_phys_lag
  real, private                              :: lag_o,lag_d
  real, private                              :: back_lag_o,back_lag_d
  real, private                              :: prop_dist,weight
  real, private                              :: wavelet_o,wavelet_d,t_o,t_d
  real, private                              :: wavelet_tdelay
  real, private                              :: back_slow_phys,dtdslow

  real, dimension(:), allocatable, private   :: data_conv
  real, dimension(:), allocatable, private   :: back_slow_conv
  real, dimension(:), allocatable, private   :: der1_wavelet,der1_wavelet_shift
  real, dimension(:), pointer, private       :: der1_wavelet_shift_p

  contains

  subroutine prop_extend_der11D_init(t_n_in,wavelet_n_in, &
      lag_n_in,lag_o_in,lag_d_in, &
      back_lag_n_in,back_lag_o_in,back_lag_d_in, &
      wavelet_o_in,wavelet_d_in,t_o_in,t_d_in,wavelet_tdelay_in,prop_dist_in)
    integer                        :: t_n_in,wavelet_n_in,lag_n_in,back_lag_n_in
    real                           :: lag_o_in,lag_d_in,prop_dist_in
    real                           :: back_lag_o_in,back_lag_d_in
    real                           :: wavelet_o_in,wavelet_d_in,t_o_in,t_d_in
    real                           :: wavelet_tdelay_in

    t_n = t_n_in
    wavelet_n = wavelet_n_in
    lag_n = lag_n_in
    lag_o = lag_o_in
    lag_d = lag_d_in
    back_lag_n = back_lag_n_in
    back_lag_o = back_lag_o_in
    back_lag_d = back_lag_d_in
    wavelet_o=wavelet_o_in
    wavelet_d=wavelet_d_in
    t_o=t_o_in
    t_d=t_d_in
    wavelet_tdelay=wavelet_tdelay_in
    prop_dist = prop_dist_in

    dtdslow=-prop_dist
!    weight=1./sqrt(float(back_lag_n))
    weight=1.

    call mem_alloc(back_slow_conv, back_lag_n-1)
    call mem_alloc(der1_wavelet, wavelet_n)
    call mem_alloc(der1_wavelet_shift, t_n)
    call mem_alloc(data_conv, t_n)
    allocate(der1_wavelet_shift_p(t_n))

    if (wavelet_d .ne. t_d) then
      write(0,*)'t_d=',t_d
      write(0,*)'wavelet_d=',wavelet_d
      call seperr("wavelet_d must be equal to t_d_data")
    end if

    back_slow_extend = back_lag_n >1
    if (back_slow_extend) then
!      if (amod(back_lag_o,back_lag_d) .ne. 0.) & 
      if (mod(back_lag_o,back_lag_d) .gt. tiny(back_lag_o)) &
        call seperr("back_lag_o mus be a multiple of back_lag_d")
      back_zero_lag=nint(-back_lag_o/back_lag_d+1) 
write(0,*)'back_zero_lag=',back_zero_lag
      if ((back_zero_lag < 2) .or. (back_zero_lag > back_lag_n)) &
        call seperr("back_zero_lag not within lag interval")
    else
      call seperr("Why call prop_extend_1D when  backslowness is not extended?")
    end if

    if (lag_n .ne. 1) then
      call seperr("lag_n /= 1 in prop_extend_der1_1D")
    end if
  end subroutine

  integer function prop_extend_der11D_back_slow_update(back_slow_phys_in)
    real                :: back_slow_phys_in
    call prop_extend_der11D_back_slow_update2(back_slow_phys_in)
    prop_extend_der11D_back_slow_update = 0
  end function

  subroutine prop_extend_der11D_back_slow_update2(back_slow_phys_in)
    real                :: back_slow_phys_in
    back_slow_phys = back_slow_phys_in
  end subroutine

  integer function prop_extend_der11D_back_slow_conv_update(back_slow_conv_in)
    real, dimension(:)   :: back_slow_conv_in
    call prop_extend_der11D_back_slow_conv_update2(back_slow_conv_in)
    prop_extend_der11D_back_slow_conv_update = 0
  end function

  subroutine prop_extend_der11D_back_slow_conv_update2(back_slow_conv_in)
    real, dimension(:)   :: back_slow_conv_in
    back_slow_conv = back_slow_conv_in
    back_slow_conv(back_zero_lag)=0.
  end subroutine

  integer function prop_extend_der11D_der1_wavelet_update(wavelet_in) 
    real, dimension(:)  :: wavelet_in
    call prop_extend_der11D_der1_wavelet_update2(wavelet_in)
    prop_extend_der11D_der1_wavelet_update=0
  end function

  subroutine prop_extend_der11D_der1_wavelet_update2(wavelet_in) ! Copy wavelet
    real, dimension(:)   :: wavelet_in
!    call igrad1_lop2(F,F,wavelet_in,der1_wavelet) 
    call igrad1_cent_lop2(F,F,wavelet_in,der1_wavelet) 
    der1_wavelet = der1_wavelet/wavelet_d
  end subroutine

  integer function prop_extend_der11D_op(adj, add, model, data)
    logical            :: adj, add
    real,dimension(:)  :: model, data
    call prop_extend_der11D_op2(adj, add, model, data)
    prop_extend_der11D_op = 0
  end function

  subroutine prop_extend_der11D_op2(adj, add, model, data)
    logical                   :: adj, add
    real, dimension(:) :: model, data
    real                   :: time_prop

    call adjnull (adj,add,model,data )

    if(adj) then
      time_prop=(prop_dist*back_slow_phys)-(wavelet_tdelay+t_o-back_lag_o)
      call lint1_stat_init (wavelet_o,t_d,-time_prop)
      call lint1_stat_lop2 (F,F, der1_wavelet, der1_wavelet_shift)
      der1_wavelet_shift_p=der1_wavelet_shift
      call tcai1_nb_init(der1_wavelet_shift_p)
      call tcai1_nb_lop2(F,F,back_slow_conv,data_conv)
      model(lag_n)= model(lag_n) + &
        dot_product(data_conv,data)*weight*dtdslow
    else
      time_prop=(prop_dist*back_slow_phys)-(wavelet_tdelay+t_o-back_lag_o)
      call lint1_stat_init (wavelet_o,t_d,-time_prop)
      call lint1_stat_lop2 (F,F, der1_wavelet, der1_wavelet_shift)
      der1_wavelet_shift_p=der1_wavelet_shift
      call tcai1_nb_init(der1_wavelet_shift_p)
      call tcai1_nb_lop2(F,F,back_slow_conv,data_conv)
      data=data + data_conv * model(lag_n)*weight*dtdslow
    end if
  end subroutine

end module
