module stepper_brent_mod
! Brent stepper
! Last edited by Ali Almomin 08/24/2015

  use stepper_obj_mod
  use prob_obj_mod

  private
  public :: stepper_brent

  type, extends(stepper_obj) :: stepper_brent ! extend line search to brents method
    !Variable extension
    real, dimension(:), allocatable   :: modl                 !
    contains
    procedure                :: run
    procedure                :: shift3
  end type

  contains

  integer function shift3(this,a,b,c,d)
    !interface
    class(stepper_brent)            :: this
    double precision           :: a
    double precision           :: b
    double precision           :: c
    double precision           :: d
    !end interface
    a = b
    b = c
    c = d
    shift3 = 0
  end function

  !Calling function to choose and run a solver
  integer function run(this, modl, prob, dmodl, alpha, success)
    !--------------------input interface functions -------
    class(stepper_brent)              :: this
    class(prob_obj)                     :: prob
    real,dimension(:)                 :: dmodl           ! search direction (input)
    real,dimension(:)                 :: modl            ! model (input/output)
    double precision                  :: alpha           ! step size (output)
    logical                           :: success
    integer                           :: stat
    integer                                          :: ITMAX
    double precision                                 :: CGOLD
    double precision                                 :: a,b,ax,cx
    double precision                                 :: d,etemp,fu,fv,fw,fx,e
    double precision                                 :: p,q,r,tol1,tol2,u,v,w,x,xm
    double precision                                 :: ZEPS
    CGOLD=0.3819
    ZEPS = 1.e-16

    if(.not. allocated(this%modl)) allocate(this%modl(prob%nmodl))
    success = .false.

    if(ax < cx) then
      a = ax
      b = cx
    else
      a = cx
      b = ax
    end if
    v=bx
    w=bx
    x=bx
    do iter=0 , ITMAX, 1
      xm=0.5*(a+b)
      tol2=2.0*(tol*abs(x)+ZEPS)
      ! Test if done
      if(abs(x-xm) <= tol2 - 0.5*(b-a)) then
        fmin = fx
        exit
      end if
      ! Construct a trial parabolic fit
      if(abs(e) > tol1) then
        r=(x-w)*(fx-fv)
        q=(x-v)*(fx-fw)
        p=(x-v)*q-(x-w)*r
        q=2.0*(q-r)
        if (q > 0.0) then
          p = -p
        end if
        q=abs(q)
        etemp=e
        e=d
        if (abs(p) >= abs(0.5*q*etemp) .or. p <= q*(a-x) .or. p >= q*(b-x)) then
          if( x>xm) then
            e = a-x
          else
            e = b-x
          end if
            d=CGOLD*e
          else
            d=p/q   ! Take the parabolic step.
            u=x+d
          if (u-a < tol2 .or. b-u < tol2) d=sign(tol1,xm-x)
        end if
      else
        if( x>xm) then
          e = a-x
        else
          e = b-x
        end if
        d=CGOLD*e
      end if
      if(abs(d) > tol1) then
        u = x+d
      else
        u = x+sign(tol1,d)
      end if
      ! only one function evaluation per iteration
      this%modl = modl + u*dmodl
      stat = prob%get_obj(this%modl, fu)
      if (fu <= fx) then
        if (u >= x) then
          a=x
        else
          b=x
        end if
        stat=this%shift3(v,w,x,u)
        stat=this%shift3(fv,fw,fx,fu)
      else
        if (u < x) then
          a=u
        else
          b=u
        end if
      end if
      if (fu <= fw .or. w == x)  then
        v=w
        w=u
        fv=fw
        fw=fu
      else if (fu <= fv .or. v == x .or. v == w) then
        v=u
        fv=fu
      end if
      alpha = x  ! update alpha
    end do
    modl = modl + alpha*dmodl
    success = .true.
    run = 0
  end function

end module
