module stepper_exhaust_mod
! Sampling stepper
! Last edited by Joe Jennings 07/24/2016

  use sep
  use stepper_obj_mod
  use prob_obj_mod

  implicit none

  private
  public :: stepper_exhaust

  type, extends(stepper_obj)                    :: stepper_exhaust ! extend line search to back tracking method
  !Variable extension
    integer                                     :: na = 1000
    real                                        :: oa = 0.0
    real                                        :: da = 0.0000000001
    double precision                            :: alpha = 0.
    character(256)                              :: plot = ''
    logical                                     :: verb = .false.
    logical                                     :: ext  = .false. ! called out of solver
    logical                                     :: time = .false.
    real, dimension(:), allocatable             :: modl
    real, dimension(:), allocatable             :: res
    real, dimension(:), allocatable             :: dres
    double precision, dimension(:), allocatable :: f
  contains
    procedure                                   :: run
  end type

  interface stepper_exhaust
    module procedure const
  end interface

  contains

  function const(na,oa,da,plot,verb,ext,time) result(this)
    optional                          :: na, oa, da, plot, verb, ext, time
    integer                           :: na
    real                              :: oa
    real                              :: da
    character(256)                    :: plot
    logical                           :: verb
    logical                           :: ext
    logical                           :: time
    type(stepper_exhaust)             :: this
    if(present(na   ))  this%na    = na 
    if(present(oa   ))  this%oa    = oa
    if(present(da   ))  this%da    = da
    if(present(plot ))  this%plot  = plot
    if(present(verb ))  this%verb  = verb
    if(present(ext  ))  this%ext   = ext
    if(present(time  )) this%time  = time
    allocate(this%f(this%na))
  end function

  !Calling function to choose and run a solver
  integer function run(this, modl, prob, dmodl, alpha, success)
    class(stepper_exhaust)            :: this
    real,dimension(:)                 :: modl            ! model (input/output)
    class(prob_obj)                   :: prob
    real,dimension(:)                 :: dmodl           ! search direction (input)
    real,dimension(1)                 :: ialpha
    real,allocatable,dimension(:)     :: fr
    double precision                  :: alpha           ! step size (output)
    logical                           :: success
    integer                           :: stat
    double precision                  :: f0
    integer                           :: ia
    integer(kind=8)                   :: timer0, timer1, rate
    real                              :: timer_sum

    if(.not. allocated(this%modl)) allocate(this%modl(prob%nmodl))
    if(.not. allocated(this%res)) allocate(this%res(prob%ndata))
    if(.not. allocated(this%dres)) allocate(this%dres(prob%ndata))
    success = .false.
    stat = prob%get_obj(modl, f0)
    alpha = this%alpha

    ! run the stepper
    call system_clock(timer0, rate)
    do ia=1,this%na
      alpha = this%oa + (ia-1)*this%da
      this%modl = modl + alpha*dmodl
      stat = prob%get_obj(this%modl, this%f(ia))
      if(this%verb) write(0,*) "ia= ",ia," alpha= ",alpha,"f= ",this%f(ia)
    end do
    ! Print time information
    if(this%time) then
      call system_clock(timer1)
      timer_sum = 1.*(timer1-timer0)/rate
      write(0,*) "stepper timer sum (sec)", timer_sum
    endif
    if(minval(this%f) < f0) then
      ialpha = minloc(this%f)
      alpha = this%oa + (ialpha(1)-1)*this%da
      success = .true.
    endif
    if(success) then
      ! finish line search, update model
      this%alpha = alpha
      modl = modl + alpha*dmodl
    end if
    if(prob%get_counter() .eq. 1 .or. this%ext) then
      if(this%plot .ne. '') then
        call to_history("n1",this%na,this%plot)
        call to_history("o1",this%oa,this%plot)
        call to_history("d1",this%da,this%plot)
        call srite(this%plot, sngl(this%f), 4*size(this%f))
      endif
    endif
    if(this%plot .ne. '' .and. .not. this%ext) then
      call to_history("n2",prob%get_counter(),this%plot)
      call srite(this%plot, sngl(this%f), 4*size(this%f))
    endif
    run = 0
  end function

end module
