clc
clear all
close all

disp('Computing results')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters and model some data
alpha = 25;
N = 1001;
dt = 0.008;
l = 4000;
s_bar = 0.001;
delay = l*s_bar;

[dr,t] = ricker(alpha,N,dt,delay);
%//////////////////////////////////////////////////////////////////////////

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform scan

% Define c0 = 0
c0 = zeros(2*N-1,1);

% Array of starting models
s0 = 0.0002:0.00001:0.0019;
s0 = s0';
n = size(s0);
n = n(1,1);

% Allocate array to store the obj scan
objgrad = zeros(n,1);
ds = 1e-12;

for i=1:n
    
    [grad_s,grad_c] = J0_grad(s0(i,1),c0,l,alpha,dt,dr);
    t1 = ltilda(s0(i,1),-grad_c,l,alpha,dt);
    
    Fgrad_c = Ffocus(grad_c);
    t2 = ltilda(s0(i,1)-ds,-Fgrad_c,l,alpha,dt);
    x1 = (t1 - t2)'*(t1 - t2);
    
    t2 = ltilda(s0(i,1)+ds,-Fgrad_c,l,alpha,dt);
    x2 = (t1 - t2)'*(t1 - t2);
    
    objgrad(i,1) = (x2-x1)/(2*ds);
    
end
%//////////////////////////////////////////////////////////////////////////

f1 = figure('units','normalized','outerposition',[0 0 1 1]);
plot(s0*1000,objgrad,':b*','LineWidth',2);
grid on;
hold on;
title('Gradient of J_s(s) at s=s_0, for each starting slowness s_0','Fontsize',15);
xlim([0 2])
ylabel('grad(J_s(s_0))','Fontsize',15);
xlabel('s_0 (ms/m)','Fontsize',15);
set(gca,'FontSize',15)
legend('grad(J_s(s_0))');
print('rahulfig_test/Jsgrad','-depsc')

close all

disp('Finished computing')