clc
clear all
close all

disp('Computing results')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set parameters and model some data
alpha = 25;
N = 1001;
dt = 0.008;
l = 4000;
s_bar = 0.001;
delay = l*s_bar;

[dr,t] = ricker(alpha,N,dt,delay);
%//////////////////////////////////////////////////////////////////////////

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform scan

% Define c0 = 0
c0 = zeros(2*N-1,1);

% Array of starting models
s0 = 0.0002:0.00001:0.0019;
s0 = s0';
n = size(s0);
n = n(1,1);

% Array of models for scan
n1 = 10; % Number of perturbations in each direction about s0
ds = 0.000001;
s = zeros(n,2*n1+1);
for i=1:n
    for j=1:(2*n1+1)
        s(i,j) = s0(i,1) + ds*(j - n1 - 1);
    end
end

% Allocate array to store the obj scan
objscan = zeros(n,n1);
objscan1 = zeros(n,n1);
objscan2 = zeros(n,n1);

for i=1:n
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Calculate grad_c, grad_c1

    grad_c = zeros(1,2*N-1);
    grad_c1 = zeros(1,2*N-1);
    
    d = ltilda(s0(i,1),grad_c',l,alpha,dt);
    temp = wrev(d');
    temp1 = zeros(1,2*N-1);
    for i1 = 1:N
        temp1 = temp1*0;
        temp1(1,i1:i1+N-1) = temp;
        grad_c = grad_c - dr(i1,1)*temp1;
        grad_c1 = grad_c1 - d(i1,1)*temp1;
    end

    grad_c = grad_c';
    grad_c1 = grad_c1';
    %//////////////////////////////////////////////////////////////////////////
    
    [t1data,t1d,t1conv] = ltilda(s0(i,1),grad_c,l,alpha,dt);
    [t3data,t3d,t3conv] = ltilda(s0(i,1),grad_c1,l,alpha,dt);
    
    Fgrad_c = Ffocus(grad_c);
    Fgrad_c1 = Ffocus(grad_c1);
    
    for j=1:(2*n1+1)
        [t2data,t2d,t2conv] = ltilda(s(i,j),Fgrad_c,l,alpha,dt);
        [t4data,t4d,t4conv] = ltilda(s(i,j),Fgrad_c1,l,alpha,dt);
        
        objscan(i,j) = 0.5*(t1conv - t2conv)'*(t1conv - t2conv);
        objscan1(i,j) = 0.5*(t3conv - t4conv)'*(t3conv - t4conv);
        objscan2(i,j) = 0.5*(t3conv - t4conv - t1conv + t2conv)'*(t3conv - t4conv - t1conv + t2conv);
    end
    
end
%//////////////////////////////////////////////////////////////////////////

disp('Finished computing')